/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_KqgZx60E_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_q0qMoGPd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1pXNfgsB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PqESeKOo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_pYswKq0a_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_X22x0PXW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_arkrVPY0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_DdTcnBnM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KTYfeplT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VLyC7JWI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_yW6F7LYE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8l03Hckt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UBiLi1XH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_v7CTJyqm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_M0aOVtrT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gL0wl98S_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_3GkAcI3y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_3ncb1LHN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_XMxJyFti_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AAKg06i1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_y2wScV4q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dw4LNDcx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_joVxtESG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_lDfxNvDv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_rh9SUR7I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0DhpYeBY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_w9lNMEAC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DgUxpgLQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iEbyAYFt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hLCtrZ5c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8ngmArN4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_E7n4VzsX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_tKemp0oL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_yUCZdDnA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_69Esm2jy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_4F5P2fRz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_650Px4kG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pBgmpDL2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IKIb8Pdb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zu7MFIx5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Kr3PjxcO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_L0mXZ06g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_80pcluA5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5PAd3tz3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rx3oZLGr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_zRDA9qwK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_FU6LpAag_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_LZrNwAbd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_sMdSN3K7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ioSiphOL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gd74Pmdf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_cmPAizA1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_zHzKymqL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_u2DDudfG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_GtlOcmRL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_R7bW8j08_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_0v0oJUn1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_6LwSvDo7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_wEtlI8av_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_54TVQC4T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gxTfmhUk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LtYWvRq7_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_Cax6TT8a;
  SignalEnvelope sEnv_3ncb1LHN;
  SignalEnvelope sEnv_XMxJyFti;
  SignalEnvelope sEnv_69Esm2jy;
  SignalEnvelope sEnv_4F5P2fRz;
  SignalLine sLine_YCB62r1v;
  SignalLine sLine_BJQIdF1K;
  ControlBinop cBinop_KqgZx60E;
  ControlBinop cBinop_q0qMoGPd;
  ControlBinop cBinop_1pXNfgsB;
  ControlBinop cBinop_PqESeKOo;
  ControlBinop cBinop_arkrVPY0;
  ControlBinop cBinop_KTYfeplT;
  ControlBinop cBinop_VLyC7JWI;
  ControlBinop cBinop_yW6F7LYE;
  ControlVar cVar_8l03Hckt;
  ControlVar cVar_v7CTJyqm;
  SignalVarf sVarf_J157HZIA;
  ControlVar cVar_M0aOVtrT;
  ControlIf cIf_gL0wl98S;
  ControlIf cIf_3GkAcI3y;
  ControlBinop cBinop_AAKg06i1;
  ControlBinop cBinop_dw4LNDcx;
  ControlBinop cBinop_joVxtESG;
  ControlBinop cBinop_0DhpYeBY;
  ControlBinop cBinop_DgUxpgLQ;
  SignalVarf sVarf_dc3SKzds;
  ControlVar cVar_E7n4VzsX;
  ControlIf cIf_tKemp0oL;
  ControlIf cIf_yUCZdDnA;
  ControlBinop cBinop_650Px4kG;
  ControlBinop cBinop_IKIb8Pdb;
  ControlBinop cBinop_zu7MFIx5;
  ControlBinop cBinop_80pcluA5;
  ControlBinop cBinop_rx3oZLGr;
  ControlVar cVar_sMdSN3K7;
  ControlBinop cBinop_ioSiphOL;
  SignalVarf sVarf_DYJ51Q01;
  SignalVarf sVarf_0WO0F07v;
  SignalVarf sVarf_H1puSB7c;
  SignalVarf sVarf_L7vckI7X;
  ControlBinop cBinop_u2DDudfG;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
